/**
 * Szablon funkcji lexical_cast suy do bezpiecznej i eleganckiej konwersji
 * wartoci w postaci cigu znakw na zmienne typu int, float, bool itp. Jest to mechanizm
 * zastpujcy funkcje atox() z jzyka c. Istnieje te moliwo konwersji typw liczbowych
 * lub boolowskich na cigi znakw.
 */

#include <string>
#include <iostream>

#include <boost/lexical_cast.hpp>

using std::string;
using std::cout;
using std::endl;
using std::ostream;
using std::istream;

using boost::lexical_cast;
using boost::bad_lexical_cast;

// Testuje lexical_cast dla wlasnej klasy
void testLcForOwnClass();

int main(int argc, char *argv[]) {
    // Utworzenie acucha znakw "1337" a nastpnie zamiana na liczb 
    // cakowit (int)
    string intStr("1337");
    int intVal = lexical_cast<int>(intStr);
    cout << "Wczytana wartosc typu int = " << intVal << endl;
    
    // Utworzenie acucha znakw "31.337" a nastpnie zamiana na liczb 
    // rzeczywist (float)
    string floatStr("31.337");
    float floatVal = lexical_cast<float>(floatStr);
    cout << "Wczytana wartosc typu float = " << floatVal << endl;

    // Konwersji do zmiennej typu bool mona dokona z cigw znakw
    // "0", "1", "0.0", "1.0" a take pojedynczych znakw '0' i '1'.
    // Niestety w biecej wersji (1.35.0) nie ma moliwoci zamiany 
    // napisw "true" i "false". Wypisanie zmiennej typu bool
    // daje w wyniki "1" dla true i "0" dla false.
    
    // Utworzenie acucha znakw "1" a nastpnie zamiana na zmienn typu
    // bool
    string trueStr("1");
    bool trueBool = lexical_cast<bool>(trueStr);
    cout << "Wczytano wartosc typu bool = "<< trueBool << endl;

    // Utworzenie pojedynczego znaku '0' a nastpnie zamiana na zmienn typu
    // bool

    char falseChar = '0';
    bool falseBool = lexical_cast<bool>(falseChar);
    cout << "Wczytano wartosc typu bool = "<< falseBool << endl;


    // Jeli konwersja si nie powiedzie, wyrzucany jest wyjtek
    // boost::bad_lexical_cast
    // Prba konwersji cigu znakw "blah" na liczb cakowit
    try {
        lexical_cast<int>("Test");
    } catch (bad_lexical_cast &e) {
        cout<<"Nieudana konwersja ciagu znakow \"Test\" na zmienna typu int, " << 
            "komunikat z wyjatku: " << e.what() << endl;
    }

    // Uwaga na puapk: konwersja do typu "char" lub "unsigned char" 
    // traktuje typ "char" lub "unsigned char" jako znakowy, nie numeryczny!
    // Operacja taka wczytuje bajt ze rda, jeli jednak ma ono wicej bajtw
    // jest wyrzucany wyjtek. Sytuacja taka moe te wystpi gdy sprbujemy 
    // rzutowa na typ uint8_t, ktry jest zwykle zadeklarowany jako unsigned char.
    try {
        lexical_cast<unsigned char>("100");
    } catch (bad_lexical_cast &e) {
        cout << "Nieudana konwersja ciagu znakow \"100\" na zmienna " <<
            "typu unsigned char, komunikat z wyjatku: "<< e.what() << endl;
    }


    // Mona te dokona konwersji "w odwrotn stron", to znaczy zamieni
    // liczb na cig znakw lub pojedynczy znak.
    string year1970(lexical_cast<string>(1970));
    cout << "Wczytano string = " << year1970 << endl;

    char luckyNumber = (lexical_cast<char>(7));
    cout << "Wczytano char = " << luckyNumber << endl;
    
    // Na koniec testujemy lexical_cast dla wlasnej klasy
    testLcForOwnClass();

}

// Jak sprawi, by lexical_cast dziaa dla naszych klas?
// Zamy, e nasza klasa Integer opakowujca liczby naturalne 
// ma by celem (target) konwersji.
class Integer {
public:
    // Musi istnie moliwo domylnego skonstruowania obiektu 
    Integer(int value=0) : value_(value) {}
    
    // Musi istnie konstruktor kopiujcy (w tym przykadzie pozwalamy aby 
    // wygenerowa go kompilator)
    //

    // Musi by zadeklarowany operator >> jako parametry przyjmujcy 
    // istream lub wstream po lewej stronie, a po prawej instancj typu 
    // bdcego rezultatem konwersji.
    friend istream& operator>>(istream& is, Integer& i) {
        return is >> i.value_;
    }

    /**
     * Zwraca warto opakowywan przez Integer
     */
    int getValue() {
        return value_;
    }
private:
    /** 
     * Warto opakowywana
     */
    int value_;
};

void testLcForOwnClass() {
    Integer i;
    
    i = lexical_cast<Integer>("234");
    cout << "Wczytano wartosc Integer = " << i.getValue() << endl;
}

